@@ -1,3 +1,4 @@
+import copy
 import csv
 from collections import defaultdict
 from glob import glob
@@ -84,27 +85,41 @@ def load_spiro(filename):
     """
     Loads COSMED spirometer data from Excel file.
 
-    Loads spirometer data to a pandas DataFrame, converts time to seconds (not datetime), computes power from the energy
-    expenditure, computes weights from the time difference between samples, if no heart rate data is available it fills
-    the column with np.NaNs, also includes VO2 and VCO2. Returns a DataFrame with:
+    Loads spirometer data to a pandas DataFrame, converts time to seconds (not datetime), computes energy expenditure,
+    computes weights from the time difference between samples, if no heart rate data is available it fills
+    the column with np.NaNs. Returns a DataFrame with:
 
-    +------------+----------------------+-----------+
-    | Column     | Data                 | Unit      |
-    +============+======================+===========+
-    | time       | time at breath       | s         |
-    +------------+----------------------+-----------+
-    | power      | power at breath      | N         |
-    +------------+----------------------+-----------+
-    | weights    | sample weight        |           |
-    +------------+----------------------+-----------+
-    | HR         | heart rate           | bpm       |
-    +------------+----------------------+-----------+
-    | RER        | exchange ratio       | VCO2/VO2  |
-    +------------+----------------------+-----------+
-    | VO2        | oxygen               | [mL/min]  |
-    +------------+----------------------+-----------+
-    | VCO2       | carbon dioxide       | [mL/min]  |
-    +------------+----------------------+-----------+
+    +------------+----------------------------+-----------+
+    | Column     | Data                        | Unit     |
+    +============+============================+===========+
+    | time       | time at breath             | s         |
+    +------------+----------------------------+-----------+
+    | HR         | heart rate                 | bpm       |
+    +------------+----------------------------+-----------+
+    | EE         | energy expenditure         | J/s       |
+    +------------+----------------------------+-----------+
+    | RER        | exchange ratio             | VCO2/VO2  |
+    +------------+----------------------------+-----------+
+    | VO2        | oxygen                     | l/min     |
+    +------------+----------------------------+-----------+
+    | VCO2       | carbon dioxide             | l/min     |
+    +------------+----------------------------+-----------+
+    | VE         | ventilation                | l/min     |
+    +------------+----------------------------+-----------+
+    | VE/VO2     | ratio VE/VO2| -            |           |
+    +------------+----------------------------+-----------+
+    | VE/VCO2    | ratio VE/VCO2              | -         |
+    +------------+----------------------------+-----------+
+    | O2pulse    | oxygen pulse  (VO2/HR)     | -         |
+    +------------+----------------------------+-----------+
+    | PetO2      | end expiratory O2 tension  | mmHg      |
+    +------------+----------------------------+-----------+
+    | PetCO2     | end expiratory CO2 tension | mmHg      |
+    +------------+----------------------------+-----------+
+    | VT         | tidal volume               | l         |
+    +------------+----------------------------+-----------+
+    | weights    | sample weight              | -         |
+    +------------+----------------------------+-----------+
 
     Parameters
     ----------
@@ -119,12 +134,18 @@ def load_spiro(filename):
     """
     data = pd.read_excel(filename, skiprows=[1, 2], usecols="J:XX")
     data["time"] = data.apply(lambda row: pd_dt_to_s(row["t"]), axis=1)  # hh:mm:ss to s
-    data["power"] = data["EEm"] * 4184 / 60  # added power (kcal/min to J/s)
+    data["EE"] = data["EEm"] * 4184 / 60  # kcal/min to J/s
     data["weights"] = np.insert(np.diff(data["time"]), 0, 0)  # used for calculating weighted average
+    data["VO2"] = data["VO2"] / 1000  # to l/min
+    data["VCO2"] = data["VCO2"] / 1000  # to l/min
     data["RER"] = data["VCO2"] / data["VO2"]
     data["HR"] = np.NaN if "HR" not in data else data["HR"]  # missing when sensor is not detected
+    data["O2pulse"] = data["VO2"] / data["HR"]
+    data["VE/VO2"] = data["VE"] / data["VO2"]
+    data["VE/VCO2"] = data["VE"] / data["VCO2"]
+
     data = data[data["time"] > 0]  # remove "missing" data
-    return data[["time", "HR", "power", "RER", "VO2", "VCO2", "weights"]]
+    return data[["time", "HR", "EE", "RER", "VO2", "VCO2", "VE", "VE/VO2", "VE/VCO2", "O2pulse", "PetO2", "PetCO2", "VT", "weights"]]
 
 
 def load_opti(filename, rotate=True):
@@ -480,7 +501,7 @@ def load_n3d(filename, verbose=True):
     return optodata
 
 
-def load_imu(root_dir, filenames=None):
+def load_imu(root_dir, filenames=None, inplace=False):
     """
     Imports NGIMU session in nested dictionary with all devices and sensors.
 
@@ -495,8 +516,8 @@ def load_imu(root_dir, filenames=None):
 
     Returns
     -------
-    session_data : dict
-        returns nested object sensordata[device][sensor][dataframe]
+    sessiondata : dict
+        returns nested object sensordata[device][dataframe]
 
     References
     ----------
@@ -507,7 +528,7 @@ def load_imu(root_dir, filenames=None):
     if not directory_contents:
         raise Exception("No contents in directory")
     directories = glob(f"{root_dir}/*/")  # folders of all devices
-    session_data = dict()
+    sessiondata = dict()
     if not filenames:
         filenames = ["sensors"]
 
@@ -517,7 +538,9 @@ def load_imu(root_dir, filenames=None):
         device_name = "left" if "links" in device_name.lower() or "left" in device_name.lower() else device_name
         device_name = "right" if "rechts" in device_name.lower() or "right" in device_name.lower() else device_name
         device_name = "frame" if "frame" in device_name.lower() else device_name
-        session_data[device_name] = dict()
+        device_name = 'trunk' if 'trunk' in device_name.lower() else device_name
+
+        sessiondata[device_name] = dict()
 
         for sensor_file in sensor_files:  # loop through all csv files
             sensor_name = path.split(sensor_file)[-1].split(".csv")[0]  # sensor without path or extension
@@ -525,14 +548,29 @@ def load_imu(root_dir, filenames=None):
             if sensor_name not in filenames:
                 continue  # skip if filenames is given and sensor not in filenames
 
-            session_data[device_name][sensor_name] = pd.read_csv(sensor_file).drop_duplicates()
-            new_col_names = session_data[device_name][sensor_name].columns
+            sessiondata[device_name][sensor_name] = pd.read_csv(sensor_file).drop_duplicates()
+            new_col_names = sessiondata[device_name][sensor_name].columns
             new_col_names = [col.lower().replace(" ", "_").rsplit("_", 1)[0] for col in new_col_names]
-            session_data[device_name][sensor_name].columns = new_col_names
+            sessiondata[device_name][sensor_name].columns = new_col_names
 
-        if not session_data[device_name]:
+        if not sessiondata[device_name]:
             raise Exception("No data was imported")
-    return session_data
+
+    if not inplace:
+        sessiondata = copy.deepcopy(sessiondata)
+
+    sessiondata["right"] = sessiondata["right"]["sensors"] if 'right' in sessiondata.keys() else print(
+        'No right sensor imported')
+    sessiondata["frame"] = sessiondata["frame"]["sensors"] if 'frame' in sessiondata.keys() else print(
+        'No frame sensor imported')
+    sessiondata["left"] = sessiondata["left"]["sensors"] if 'left' in sessiondata.keys() else print(
+        'No left sensor imported')
+    sessiondata["trunk"] = sessiondata["trunk"]["sensors"] if 'trunk' in sessiondata.keys() else print(
+        'No trunk sensor imported')
+
+    sessiondata = {a: b for a, b in sessiondata.items() if b is not None}
+
+    return sessiondata
 
 
 def load_drag_test(filename):
@@ -651,9 +689,66 @@ def load_opti_offset(filename):
         Offset Optipush data in a pandas DataFrame
 
     """
-    names = ["fx", "fy", "fz", "mx", "my", "torque", "angle", "angle2"]
+    names = ["fx", "fy", "fz", "mx", "my", "torque", "angle", "angle_360"]
     opti_offset_df = pd.read_csv(filename, names=names, delimiter="\t", skiprows=12)
     opti_offset_df["angle"] *= (np.pi / 180)
     opti_offset_df["torque"] *= -1
 
     return opti_offset_df
+
+
+def load_movesense(root_dir, right, frame=None, left=None):
+    """
+    Imports MoveSense data in nested dictionary with all sensors.
+
+    Parameters
+    ----------
+    root_dir : str
+        directory where session is located
+    right : str
+        12 digit identity code of right sensor
+    frame : str
+        12 digit identity code of frame sensor
+    left : str
+        12 digit identity code of left sensor
+
+    Returns
+    -------
+    sessiondata : dict
+        returns nested object sensordata[device][dataframe]
+
+
+    """
+    sessiondata = dict()
+
+    right_sensors = sorted(glob(root_dir + right + '*'))
+    sensors = [right_sensors]
+    if frame is not None:
+        frame_sensors = sorted(glob(root_dir + frame + '*'))
+        sensors.append(frame_sensors)
+    if left is not None:
+        left_sensors = sorted(glob(root_dir + left + '*'))
+        sensors.append(left_sensors)
+
+    for sensor in sensors:
+        if right in sensor[0]: sensor_name = 'right'
+        if frame is not None:
+            if frame in sensor[0]: sensor_name = 'frame'
+        if left is not None:
+            if left in sensor[0]: sensor_name = 'left'
+
+        acc = pd.read_csv(sensor[0])
+        acc['x'] *= -1
+        gyro = pd.read_csv(sensor[1])
+        gyro['x'] *= -1
+        acc.rename(columns={'x': 'accelerometer_y', 'y': 'accelerometer_x', 'z': 'accelerometer_z'}, inplace=True)
+        gyro.rename(columns={'x': 'gyroscope_y', 'y': 'gyroscope_x', 'z': 'gyroscope_z'}, inplace=True)
+
+        acc['time'] = pd.to_datetime(acc['timestamp'], unit='ms')
+        acc['time'] -= acc['time'][0]
+        acc['time'] = acc['time'].dt.total_seconds()
+
+        sessiondata[sensor_name] = pd.concat([gyro, acc], axis=1, join='inner')
+        sessiondata[sensor_name] = sessiondata[sensor_name].drop(['timestamp'], axis=1)
+
+    return sessiondata
\ No newline at end of file
