@@ -1,12 +1,12 @@
 import numpy as np
 import pandas as pd
 from scipy.integrate import cumtrapz
-
+from scipy.signal import savgol_filter
 from .utils import lowpass_butter, find_peaks
-
+from .move import rotate_matrix
 
 def auto_process(data, wheelsize=0.31, rimsize=0.27, sfreq=200, co_f=15, ord_f=2, co_s=6, ord_s=2, force=True,
-                 speed=True, variable="torque", cutoff=0.0, minpeak=5.0):
+                 speed=True, variable="torque", cutoff=0.0, wl=201, ord_a=2, minpeak=5.0):
     """
     Top level processing function that performs all processing steps for mw/ergo data.
 
@@ -39,6 +39,10 @@ def auto_process(data, wheelsize=0.31, rimsize=0.27, sfreq=200, co_f=15, ord_f=2
         variable name used for peak (push) detection
     cutoff : float
         noise level for peak (push) detection
+    wl : float
+        window length angle filter
+    ord_a : int
+        order angle filter [..]
     minpeak : float
         min peak height for peak (push) detection
 
@@ -57,13 +61,13 @@ def auto_process(data, wheelsize=0.31, rimsize=0.27, sfreq=200, co_f=15, ord_f=2
         data = process_ergo(data, wheelsize, rimsize)
         pushes = push_by_push_ergo(data, variable, cutoff, minpeak)
     else:
-        data = filter_mw(data, sfreq, co_f, ord_f, co_s, ord_s, force, speed)
+        data = filter_mw(data, sfreq, co_f, ord_f, wl, ord_a, force, speed)
         data = process_mw(data, wheelsize, rimsize, sfreq)
         pushes = push_by_push_mw(data, variable, cutoff, minpeak)
     return data, pushes
 
 
-def filter_mw(data, sfreq=200., co_f=15., ord_f=2, co_s=6., ord_s=2, force=True, speed=True):
+def filter_mw(data, sfreq=200., co_f=15., ord_f=2, wl=201, ord_a=2, force=True, speed=True):
     """
     Filters measurement wheel data.
 
@@ -79,10 +83,10 @@ def filter_mw(data, sfreq=200., co_f=15., ord_f=2, co_s=6., ord_s=2, force=True,
         cutoff frequency force filter [Hz]
     ord_f : int
         order force filter [..]
-    co_s : float
-        cutoff frequency force filter [Hz]
-    ord_s : int
-        order speed filter [..]
+    wl : float
+        window length angle filter
+    ord_a : int
+        order angle filter [..]
     force : bool
         force filter toggle, default is True
     speed : bool
@@ -103,7 +107,7 @@ def filter_mw(data, sfreq=200., co_f=15., ord_f=2, co_s=6., ord_s=2, force=True,
         for var in frel:
             data[var] = lowpass_butter(data[var], cutoff=co_f, order=ord_f, sfreq=sfreq)
     if speed:
-        data["angle"] = lowpass_butter(data["angle"], cutoff=co_s, order=ord_s, sfreq=sfreq)
+        data["angle"] = savgol_filter(data["angle"], window_length=wl, polyorder=ord_a)
     return data
 
 
@@ -377,11 +381,11 @@ def push_by_push_mw(data, variable="torque", cutoff=0.0, minpeak=5.0, mindist=5,
     return pbp
 
 
-def push_by_push_ergo(data, variable="torque", cutoff=0.0, minpeak=5.0, mindist=5, verbose=True):
+def push_by_push_ergo(data, variable="power", cutoff=0.0, minpeak=50.0, mindist=5, verbose=True):
     """
     Push-by-push analysis for wheelchair ergometer data.
 
-    Push detection and push-by-push analysis for measurement wheel data. Returns a pandas DataFrame with:
+    Push detection and push-by-push analysis for ergometer data. Returns a pandas DataFrame with:
 
     +--------------------+----------------------+-----------+
     | Column             | Data                 | Unit      |
@@ -424,26 +428,29 @@ def push_by_push_ergo(data, variable="torque", cutoff=0.0, minpeak=5.0, mindist=
     data : dict
         wheelchair ergometer dictionary
     variable : str
-        variable name used for peak (push) detection
+        variable name used for peak (push) detection, default = power
     cutoff : float
-        noise level for peak (push) detection
+        noise level for peak (push) detection, default = 0
     minpeak : float
-        min peak height for peak (push) detection
+        min peak height for peak (push) detection, default = 50.0
     mindist : int
         minimum sample distance between peak candidates, can be used to speed up algorithm
 
     Returns
     -------
     pbp : dict
-        dictionary with left and right push-by-push DataFrame
+        dictionary with left, right and mean push-by-push DataFrame
     """
-    pbp_sides = {"left": [], "right": []}
+    pbp_sides = {"left": [], "right": [], "mean": []}
     keys = ["start", "stop", "peak", "tstart", "tstop", "tpeak", "cangle", "cangle_deg", "ptime", "meanpower",
             "maxpower", "meantorque", "maxtorque", "meanuforce", "maxuforce", "meanforce", "maxforce", "work",
             "slope", "smoothness", "ctime", "reltime", "cwork", "negwork"]
 
     for side in data:
-        peaks = find_peaks(data[side][variable], cutoff, minpeak, mindist)
+        if (side == 'left') | (side == 'right'):
+            peaks = find_peaks(data[side][variable], cutoff, minpeak, mindist)
+        if (side == 'mean'):
+            peaks = find_peaks(data[side][variable], cutoff, (minpeak*2), mindist)
         pbp = pd.DataFrame(data=np.full((len(peaks["start"]), len(keys)), np.NaN), columns=keys)  # preallocate
 
         pbp["start"] = peaks["start"]
@@ -481,6 +488,34 @@ def push_by_push_ergo(data, variable="torque", cutoff=0.0, minpeak=5.0, mindist=
         pbp_sides[side] = pd.DataFrame(pbp)
 
     if verbose:
-        print("\n" + "=" * 80 + f"\nFound left: {len(pbp_sides['left'])} and right: {len(pbp_sides['right'])} pushes!\n"
+        print("\n" + "=" * 80 + f"\nFound left: {len(pbp_sides['left'])} , right: {len(pbp_sides['right'])} and mean: {len(pbp_sides['mean'])} pushes!\n"
               + "=" * 80 + "\n")
     return pbp_sides
+
+
+def camber_correct(data, ang):
+    """Correct for camber angle in measurement wheel data
+
+    Parameters
+    ----------
+    data : pd.DataFrame()
+        measurement wheel data with forces/torques in 3D
+
+    ang : int
+        camber angle to correct
+
+    Returns
+    -------
+    data: pd.DataFrame()
+        measurement wheel data with forces/torques in 3D
+        corrected for camber angle
+    """
+
+    force = data[['fx', 'fy', 'fz']].T
+    torques = data[['mx', 'my', 'torque']].T
+
+    rotmat = rotate_matrix(ang/(180*np.pi), axis='x')
+    data[['fx', 'fy', 'fz']] = np.dot(rotmat, force).T
+    data[['mx', 'my', 'torque']] = np.dot(rotmat, torques).T
+
+    return data
